local Tags={}
Relog.Tags=Tags

local TagCompiler = AceLibrary("TagCompiler-1.0")

local PowerColors={}
local ClassColors={}

for i=0,4 do
	PowerColors[i]=string.format("|cff%02x%02x%02x",255*ManaBarColor[i].r,255*ManaBarColor[i].g,255*ManaBarColor[i].b)
end

for k,v in pairs(RAID_CLASS_COLORS) do
	ClassColors[k]=string.format("|cff%02x%02x%02x",255*v.r,255*v.g,255*v.b)
end

local TagData = {
	Time = { tableData = "Time" },
	Type = { tableData = "Type" },
	Source = { tableData = "Source" },
	Victim = { tableData = "Victim" },
	Ability = { tableData = "Ability" },
	Result = { tableData = "Result" },
	Amount = { tableData = "Amount" },

	Attribute ={ tableData = "Attribute"},

	--Leech Info
	SourceGained={ tableData = "SourceGained"},
	AttributeGained={ tableData = "AttributeGained"},
	AmountGained={ tableData = "AmountGained"},

	--Reputation
	Faction={ tableData = "Faction"},
	Rank={ tableData = "Rank"},

	--Special Data for Damage
	DamageType={ tableData = "DamageType"},
	AbsorbAmount={ tableData = "AbsorbAmount"},
	BlockAmount={ tableData = "BlockAmount"},
	ResistAmount={ tableData = "ResistAmount"},
	VulnerableAmount={ tableData = "VulnerableAmount"},

	--Healing
	Overheal={ tableData = "Overheal"},

	--Item Data
	ItemLink={ tableData = "ItemLink"},
	ItemName={ tableData = "ItemName"},
	ItemRarity={ tableData = "ItemRarity"},
	ItemString={ tableData = "ItemString"},

	--Items just for Experience which requires several
	BonusAmount={  tableData = "BonusAmount"},
	BonusType={ tableData = "BonusType"},
	PenaltyAmount={ tableData = "PenaltyAmount"},
	PenaltyType={ tableData = "PenaltyType"},
	AmountRaidPenalty={ tableData = "AmountRaidPenalty"},
	AmountGroupBonus={ tableData = "AmountGroupBonus"},


	--Unit Functions
	UnitID = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.UnitID end },
	Friendly = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Friendly end },
	isPlayer = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.isPlayer end },
	isGrouped = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.isGrouped end },
	Class = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.lClass end },
	Faction = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Faction end },
	Level = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Level end },

	Health = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Health end },
	HealthMax = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.HealthMax end },
	Power = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Power end },
	PowerMax = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.PowerMax end },
	PowerType = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.PowerType end },
	MinRange = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.MinRange end },
	MaxRange = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.MaxRange end },

	RankName = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.RankName end },
	RankNumber = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.RankNumber end },

	FriendlyC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if type(prior)~="string" then return "" end if U.Friendly then return "|cff40ff40"..prior.."|r";else return "|cffff4040"..prior.."|r" end end },
	ClassC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Class then return ClassColors[U.Class]..prior.."|r";else return prior end end },

	--Start Color Commands
	StartFriendlyC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Friendly then return "|cff40ff40";else return "|cffff4040" end end },
	StartClassC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Class then return ClassColors[U.Class];else return "" end end },
	StartPowerC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.PowerType then return PowerColors[U.PowerType];else return "" end end },

	--Date Command Same as normal except %s=Milliseconds
	Date = { method = function(tP, prior,arg1,arg2) local SubSecond=string.format("%03d",1000*(arg2-math.floor(arg2)));arg1=string.gsub(arg1,"%%s",SubSecond);return date(arg1,math.floor(arg2)) end,
		parameterTypes = {"string", "number"},	},
}
--\"%H:%M:%S.%s\"
local DefaultTags={
	Buff =		"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] is buffed with [Ability]([Amount])",
	Debuff =	"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] is debuffed with [Ability]([Amount])",
	Cast =		"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC] [Result:\"begins to \"][Amount:\"perform \"][Ability][Victim:\"on \"][Victim:FriendlyC]",
	Damage =	"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC..\"(\"..Health..\"/\"..HealthMax..\")\"] [Result] [Victim:FriendlyC..\"(\"..Health..\"/\"..HealthMax..\")\"] for [Amount] [DamageType:DamageType]",
	Death =		"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] killed [Source:\"by \"][Source:ClassC][Ability:\"'s \"][Ability:Ability]",
	Durability =	"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC]'s [Ability] damages [Victim:FriendlyC]'s [ItemLink]",
	Enchant =	"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC] enchants [Victim:FriendlyC]'s [ItemLink] with [Ability]",
	Environment =	"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] [Result] for [Amount] [DamageType]",
	Experience =	"[Date(\"%H:%M:%S.%s\",Time)] Exp: [Amount] from [Source:FriendlyC]([Source:Level])",	
	["Extra Attack"] = "[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] gains [Amount] extra attack[Amount>1:\"s\"] from [Ability]",
	Fade =		"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC]'s [Ability] fades",
	Fail =		"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC]'s [Ability] fails due to [Reason]",
	["Feed Pet"] =	"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] feeds their pet [ItemLink]",
	Gain =		"[Date(\"%H:%M:%S.%s\",Time)] [Victim:FriendlyC] gains [Amount] [Attribute] from [Source:FriendlyC]",
	Heal =		"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC] [Result] [Victim:FriendlyC] [Amount] [Overheal:\"(Overheal: \"][Overheal][Overheal:\")\"]",	
	Honor =		"[Date(\"%H:%M:%S.%s\",Time)] Honor: [Amount] from [Source:FriendlyC] [Source:RankName]",	
	Interrupt =	"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC] interrupts [Victim:FriendlyC]'s [Ability]",
	Item =		"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC] [Result]s [Amount]x [ItemLink]",
	Leech =		"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC] leeches [Amount] [Attribute] from [Victim:FriendlyC] to [SourceGained:FriendlyC] as [AmountGained] [AttributeGained]",
	Miss =		"[Date(\"%H:%M:%S.%s\",Time)] [Source:FriendlyC][Ability:\"'s \"][Ability:Ability] [Result] [Victim:FriendlyC]",
	Reputation =	"[Date(\"%H:%M:%S.%s\",Time)] Rep: [Ability:Ability] [Amount] with [Faction]([Rank])",
}

local CurTime
function Relog:TestTag(num)
	--Relog:Print(TagCompiler:Evaluate("[Type] [Source] [Result] [Victim]", TagData, "string", Relog:GetLog(num)))
	local log=Relog:GetLog(num)

	CurTime=log.Time
	Tags.ResetCache()

	if DefaultTags[log.Type] then
		Relog.db.char.Log.CompiledCache[num]=TagCompiler:Evaluate(DefaultTags[log.Type], TagData, "string", log)
		Relog:CustomPrint(1,1,1,ChatFrame3,nil,nil,Relog.db.char.Log.CompiledCache[num])
	else
		Relog:CustomPrint(1,1,1,ChatFrame3,nil,nil,"No String for "..log.Type)
	end
end

local UnitCache={}
local NumInCache

function Tags.ResetCache()
	for k in pairs(UnitCache) do
		UnitCache[k]=nil
	end

	NumInCache=0
end

local Previous

function Tags.GetUnit(name)
	if name==true then
		name=Previous
	else
		Previous=name
	end

	local Unit=UnitCache[name]

	if Unit==nil then
		NumInCache=NumInCache+1
		UnitCache[name]=Relog.Units:GetUnit(name,CurTime,NumInCache)
		Unit=UnitCache[name]
	end

	return Unit	
end