local Units={}
Relog.Units=Units

local MaxUpdateRate=0.2

local Roster = AceLibrary("Roster-2.1")
local rc = AceLibrary("RangeCheck-1.0")

local Abbreviate={
	UnitID=true,
	lClass=true,
	Class=true,
	Faction=true,
	PowerType=true,
	RankName=true,
}

local NewEntry={}

function Units:UpdateUnit(UnitID, Time)
	if not UnitID or not UnitExists(UnitID) then
		return
	end

	local Name=UnitName(UnitID)
	
	local Unit=Relog.db.char.Units[Name]

	if Unit.LastUpdated>(Time-MaxUpdateRate) then
		return
	end

	Unit.LastUpdated=Time

	local entryNum=math.floor(Time*100)


	NewEntry.UnitID=UnitID

	if Roster:GetUnitObjectFromUnit(UnitID) then
		NewEntry.isGrouped=true
	else
		NewEntry.isGrouped=false
	end

	NewEntry.isPlayer=UnitIsPlayer(UnitID)
	NewEntry.Friendly=UnitIsFriend(UnitID,"player")
	NewEntry.lClass, NewEntry.Class=UnitClass(UnitID)
	_, NewEntry.Faction=UnitFactionGroup(UnitID)
	NewEntry.Level=UnitLevel(UnitID)
	NewEntry.Health=UnitHealth(UnitID)
	NewEntry.HealthMax=UnitHealthMax(UnitID)
	NewEntry.Power=UnitMana(UnitID)
	NewEntry.PowerMax=UnitManaMax(UnitID)
	NewEntry.PowerType=UnitPowerType(UnitID)
	NewEntry.RankNumber=UnitPVPRank(UnitID)
	NewEntry.RankName=GetPVPRankInfo(NewEntry.RankNumber,UnitID)
	NewEntry.MinRange, NewEntry.MaxRange = rc:getRange(UnitID)

	for k, v in pairs(NewEntry) do
		if Abbreviate[k] then
			v=Relog.Abbrev:GetAbbrev(v)
		end

		if v~=Unit.Last[k] then
			Unit[k][entryNum]=v
			Unit.Last[k]=v
		end
	end
end

function Units:UpdateUnitByName(Name, Time)
	local Unit=Relog.db.char.Units[Name]
	local UnitID

	--First Check Last Unit
	local LastUnitID
	if Unit.Last.UnitID then
		LastUnitID=Relog.Abbrev:GetWord(Unit.Last.UnitID)
	end

	if LastUnitID and UnitExists(LastUnitID) and UnitName(LastUnitID)==Name then
		UnitID=LastUnitID
	else
		--Next check our target
		if Name and Name==UnitName("target") then
			UnitID="target"
		else
			--Lastly check through our raids targets
			for unit in Roster:IterateRoster(true) do
				if UnitName(unit.unitid.."target")==Name then
					UnitID=unit.unitid.."target"
					break
				end
			end
		end
	end
	
	if not UnitID or not UnitExists(UnitID) then
		return
	end

	local Name=UnitName(UnitID)

	if Unit.LastUpdated>(Time-MaxUpdateRate) then
		return
	end

	Unit.LastUpdated=Time

	local entryNum=math.floor(Time*100)


	NewEntry.UnitID=UnitID

	if Roster:GetUnitObjectFromUnit(UnitID) then
		NewEntry.isGrouped=true
	else
		NewEntry.isGrouped=false
	end

	NewEntry.isPlayer=UnitIsPlayer(UnitID)
	NewEntry.Friendly=UnitIsFriend(UnitID,"player")
	NewEntry.lClass, NewEntry.Class=UnitClass(UnitID)
	_, NewEntry.Faction=UnitFactionGroup(UnitID)
	NewEntry.Level=UnitLevel(UnitID)
	NewEntry.Health=UnitHealth(UnitID)
	NewEntry.HealthMax=UnitHealthMax(UnitID)
	NewEntry.Power=UnitMana(UnitID)
	NewEntry.PowerMax=UnitManaMax(UnitID)
	NewEntry.PowerType=UnitPowerType(UnitID)
	NewEntry.RankNumber=UnitPVPRank(UnitID)
	NewEntry.RankName=GetPVPRankInfo(NewEntry.RankNumber,UnitID)
	NewEntry.MinRange, NewEntry.MaxRange = rc:getRange(UnitID)

	for k, v in pairs(NewEntry) do
		if Abbreviate[k] then
			v=Relog.Abbrev:GetAbbrev(v)
		end

		if v~=Unit.Last[k] then
			Unit[k][entryNum]=v
			Unit.Last[k]=v
		end
	end
end

local Skip={
	LastUpdated=true,
	Last=true
}

local UnitCache={}
--Test /script Relog:PrintLiteral(Relog.Units:GetUnit("Ludith",1185057378,1))
function Units:GetUnit(Name, Time, CacheNum)
	local Unit

	if type(UnitCache[CacheNum])~="table" then
		UnitCache[CacheNum]={}
	end
	Unit=UnitCache[CacheNum]
	
	local UnitData=Relog.db.char.Units[Name]

	Time=Time*100

	local VTime
	for k, v in pairs(UnitData) do
		if not Skip[k] then
			Unit[k]=nil
			VTime=0
			for T, Val in pairs(v) do
				if T<=Time and T>VTime then
					Unit[k]=Val
					VTime=T
				end				
			end
			if Unit[k] and Abbreviate[k] then
				Unit[k]=Relog.Abbrev:GetWord(Unit[k])
			end
		end
	end
	
	return Unit
end